@@ -130,6 +130,7 @@ from __future__ import annotations
 from base64 import b64decode
 from datetime import datetime
 from enum import Enum
+from importlib.metadata import version
 import json
 import os
 from posixpath import join
@@ -249,7 +250,7 @@ class RunRequest(BaseModel):
     custom_settings: dict[str, Any] = Field(None)
     """Custom settings to override default IQM hardware settings and calibration data.
 Note: This field should be always None in normal use."""
-    calibration_set_id: Optional[int] = Field(None)
+    calibration_set_id: Optional[UUID] = Field(None)
     """ID of the calibration set to use, or None to use the latest calibration set"""
     qubit_mapping: Optional[list[SingleQubitMapping]] = Field(None)
     """mapping of logical qubit names to physical qubit names, or None if using physical qubit names"""
@@ -270,7 +271,7 @@ CircuitMeasurementResultsBatch = list[CircuitMeasurementResults]
 class Metadata(BaseModel):
     """Metadata describing a circuit execution job."""
 
-    calibration_set_id: Optional[int] = Field(None)
+    calibration_set_id: Optional[UUID] = Field(None)
     """ID of the calibration set used"""
     request: RunRequest = Field(...)
     """copy of the original RunRequest sent to the server"""
@@ -491,6 +492,10 @@ class IQMClient:
 
     Args:
         url: Endpoint for accessing the server. Has to start with http or https.
+        client_signature: String that IQMClient adds to User-Agent header of requests
+            it sends to the server. The signature is appended to IQMClients own version
+            information and is intended to carry additional version information,
+            for example the version information of the caller.
         tokens_file: Optional path to a tokens file used for authentication.
             This can also be set in the IQM_TOKENS_FILE environment variable.
             If tokens_file is set, auth_server_url, username and password
@@ -509,13 +514,21 @@ class IQMClient:
     """
 
     def __init__(
-        self, url: str, tokens_file: Optional[str] = None, **credentials  # contains auth_server_url, username, password
+        self,
+        url: str,
+        *,
+        client_signature: Optional[str] = None,
+        tokens_file: Optional[str] = None,
+        **credentials,  # contains auth_server_url, username, password
     ):
         if not url.startswith(('http:', 'https:')):
             raise ClientConfigurationError(f'The URL schema has to be http or https. Incorrect schema in URL: {url}')
         if tokens_file and credentials:
             raise ClientConfigurationError('Either external token or credentials must be provided. Both were provided.')
         self._base_url = url
+        self._signature = f'iqm-client {version("iqm-client")}'
+        if client_signature:
+            self._signature += f', {client_signature}'
         self._tokens_file = tokens_file
         self._external_token = _get_external_token(tokens_file)
         if not self._external_token:
@@ -552,7 +565,7 @@ class IQMClient:
         *,
         qubit_mapping: Optional[dict[str, str]] = None,
         custom_settings: Optional[dict[str, Any]] = None,
-        calibration_set_id: Optional[int] = None,
+        calibration_set_id: Optional[UUID] = None,
         shots: int = 1,
     ) -> UUID:
         """Submits a batch of quantum circuits for execution on a quantum computer.
@@ -599,7 +612,7 @@ class IQMClient:
             shots=shots,
         )
 
-        headers = {'Expect': '100-Continue'}
+        headers = {'Expect': '100-Continue', 'User-Agent': self._signature}
         if bearer_token:
             headers['Authorization'] = bearer_token
 
@@ -617,7 +630,7 @@ class IQMClient:
         result = self._retry_request_on_error(
             lambda: requests.post(
                 join(self._base_url, 'jobs'),
-                json=data.dict(exclude_none=True),
+                json=json.loads(data.json(exclude_none=True)),
                 headers=headers,
                 timeout=REQUESTS_TIMEOUT,
             )
@@ -627,7 +640,12 @@ class IQMClient:
             raise ClientAuthenticationError(f'Authentication failed: {result.text}')
 
         result.raise_for_status()
-        return UUID(result.json()['id'])
+
+        try:
+            job_id = UUID(result.json()['id'])
+            return job_id
+        except (json.decoder.JSONDecodeError, KeyError) as e:
+            raise CircuitExecutionError(f'Invalid response: {result.text}, {e}') from e
 
     def get_run(self, job_id: UUID) -> RunResult:
         """Query the status and results of a submitted job.
@@ -642,18 +660,25 @@ class IQMClient:
             HTTPException: http exceptions
             CircuitExecutionError: IQM server specific exceptions
         """
+        headers = {'User-Agent': self._signature}
         bearer_token = self._get_bearer_token()
+        if bearer_token:
+            headers['Authorization'] = bearer_token
 
         result = self._retry_request_on_error(
             lambda: requests.get(
-                join(self._base_url, 'jobs/', str(job_id)),
-                headers=None if not bearer_token else {'Authorization': bearer_token},
+                join(self._base_url, 'jobs', str(job_id)),
+                headers=headers,
                 timeout=REQUESTS_TIMEOUT,
             )
         )
 
         result.raise_for_status()
-        run_result = RunResult.from_dict(result.json())
+        try:
+            run_result = RunResult.from_dict(result.json())
+        except (json.decoder.JSONDecodeError, KeyError) as e:
+            raise CircuitExecutionError(f'Invalid response: {result.text}, {e}') from e
+
         if run_result.warnings:
             for warning in run_result.warnings:
                 warnings.warn(warning)
@@ -674,18 +699,25 @@ class IQMClient:
             HTTPException: http exceptions
             CircuitExecutionError: IQM server specific exceptions
         """
+        headers = {'User-Agent': self._signature}
         bearer_token = self._get_bearer_token()
+        if bearer_token:
+            headers['Authorization'] = bearer_token
 
         result = self._retry_request_on_error(
             lambda: requests.get(
-                join(self._base_url, 'jobs/', str(job_id), 'status'),
-                headers=None if not bearer_token else {'Authorization': bearer_token},
+                join(self._base_url, 'jobs', str(job_id), 'status'),
+                headers=headers,
                 timeout=REQUESTS_TIMEOUT,
             )
         )
 
         result.raise_for_status()
-        run_result = RunStatus.from_dict(result.json())
+        try:
+            run_result = RunStatus.from_dict(result.json())
+        except (json.decoder.JSONDecodeError, KeyError) as e:
+            raise CircuitExecutionError(f'Invalid response: {result.text}, {e}') from e
+
         if run_result.warnings:
             for warning in run_result.warnings:
                 warnings.warn(warning)
@@ -725,10 +757,14 @@ class IQMClient:
             APITimeoutError: time exceeded the set timeout
             ClientConfigurationError: if no valid authentication is provided
         """
+        headers = {'User-Agent': self._signature}
         bearer_token = self._get_bearer_token()
+        if bearer_token:
+            headers['Authorization'] = bearer_token
+
         result = requests.get(
             join(self._base_url, 'quantum-architecture'),
-            headers=None if not bearer_token else {'Authorization': bearer_token},
+            headers=headers,
             timeout=REQUESTS_TIMEOUT,
         )
 
@@ -742,7 +778,11 @@ class IQMClient:
             raise ClientAuthenticationError(f'Authentication failed: {result.text}')
 
         result.raise_for_status()
-        return QuantumArchitecture(**result.json()).quantum_architecture
+        try:
+            qa = QuantumArchitecture(**result.json()).quantum_architecture
+        except (json.decoder.JSONDecodeError, KeyError) as e:
+            raise CircuitExecutionError(f'Invalid response: {result.text}, {e}') from e
+        return qa
 
     def close_auth_session(self) -> bool:
         """Terminate session with authentication server if there was one created.
